;;; --------------------------------------------------------------------------
;;; 
;;; Date: ven mai 21 14:19:24 CEST 2004
;;;
;;; --------------------------------------------------------------------------
;;;
;;; (C) 2002 Philippe Brochard <hocwp@free.fr>
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;;
;;; --------------------------------------------------------------------------

(setq *default-pathname-defaults* *load-pathname*)
(setq *load-verbose* nil)



(load "net")
(load "tools")
(load "my-http")

(use-package "TOOLS")


(defvar *check* nil)
(defvar *action* t)


;; change the line above to change program name
(defvar *prog-name* "clwebchat")

;; change the line above for default directory install under UNIX
#+UNIX (defvar *default-install-dir* (format nil "/usr/local/~A" *prog-name*))
;; change the line above for default directory install under WINDOWS
#+WIN32 (defvar *default-install-dir* (format nil "c:\\Program Files\\~A" *prog-name*))

#+UNIX (defvar *launcher-name* *prog-name*)
#+WIN32 (defvar *launcher-name* (format nil "~A.bat" *prog-name*))


;; change lines above if you want to use an alternate lisp implementation
#+(and CLISP WIN32) (defvar *lisp* "@clisp -q -E ISO-8859-1")
#+(and CLISP UNIX) (defvar *lisp* "clisp -q -E ISO-8859-1")
#+CMU (defvar *lisp* "lisp -load")

#+CLISP (defvar *prog-bin-name* (format nil "~A.fas" *prog-name*))
#+CMU (defvar *prog-bin-name* (format nil "~A.x86f" *prog-name*))



;; this variables are set in function 'configure-files' (see below)
(defvar *bin-dir* nil)
(defvar *lib-dir* nil)
(defvar *doc-dir* nil)
(defvar *man-dir* nil)
(defvar *share-dir* nil)
(defvar *install-dir* nil)


(defun show-help ()
  (format t "Usage: install.lisp [options]
  Where options are:
    -h, --help            this help
        --install-dir     installation directory (default: ~A)
    -n, --check           do no action, just show what to do
    -f, --config          only perform configuration of files
    -c, --compile         only perform compilation of files
    -i, --install         only install files
    -l, --clean           only clean all compiled files~%"
	  *default-install-dir*)
  (quit))


(defun read-command-line ()
  (let ((cmd-line (get-command-line-words)))
    (do ((arg (pop cmd-line) (pop cmd-line)))
	((null arg))
      (case (intern (string-upcase arg))
	((-H --HELP) (show-help))
	(--INSTALL-DIR (setq *install-dir* (pop cmd-line)))
	((-N --CHECK) (setq *check* t))
	((-f --CONFIG) (setq *action* 'CONFIG))
	((-C --COMPILE) (setq *action* 'COMPILE))
	((-I --INSTALL) (setq *action* 'INSTALL))
	((-L --CLEAN) (setq *action* 'CLEAN))))))

(read-command-line)



(defun resume-install (&optional (say-install nil) (wait-key nil))
  (format t "Installing ~A in ~A~%" *prog-name* *install-dir*)
  (format t "  launcher script is ~A~%" *launcher-name*)
  (format t "  with lisp '~A'~%" *lisp*)
  (format t "  and binary program '~A'~%" *prog-bin-name*)

  (format t "bin pathname: ~A~%" *bin-dir*)
  (format t "lib pathname: ~A~%" *lib-dir*)
  (format t "doc pathname: ~A~%" *doc-dir*)
  (format t "man pathname: ~A~%" *man-dir*)
  (format t "share pathname: ~A~%" *share-dir*)

  (when say-install
    (format t "~%**************************************************~2%")
    (format t "To start ~A : run ~A~A~2%"
	    *prog-name*
	    *bin-dir* *launcher-name*)
    (format t "**************************************************~2%"))
  (when wait-key
    (format t "Hit a key to continue.~%")
     (read-line)))



(defun ask-for-user (msg default)
  (format t "~%**************************************************~%~%")
  (format t "~A~%(default: ~A) ? " msg default)
  (let ((line (read-line)))
    (if (string= line "")
	default
      line)))



(defun configure-files ()
  (unless *install-dir*
    (setq *install-dir*
	  (ask-for-user
	   (format nil "Where do you want to install '~A' ?" *prog-name*)
	   *default-install-dir*)))


  #+UNIX (setq *bin-dir* (format nil "~A/bin/" *install-dir*))
  #+WIN32 (setq *bin-dir* (format nil "~A\\bin\\" *install-dir*))

  #+UNIX (setq *lib-dir* (format nil "~A/lib/~A/" *install-dir* *prog-name*))
  #+WIN32 (setq *lib-dir* (format nil "~A\\lib\\~A\\" *install-dir* *prog-name*))

  #+UNIX (setq *doc-dir* (format nil "~A/share/doc/~A/" *install-dir* *prog-name*))
  #+WIN32 (setq *doc-dir* (format nil "~A\\doc\\~A\\" *install-dir* *prog-name*))

  #+UNIX (setq *man-dir* (format nil "~A/man/man1/" *install-dir*))
  #+WIN32 (setq *man-dir* (format nil "~A\\man\\" *install-dir*))

  #+UNIX (setq *share-dir* (format nil "~A/share/~A/" *install-dir* *prog-name*))
  #+WIN32 (setq *share-dir* (format nil "~A\\share\\~A\\" *install-dir* *prog-name*))

  (resume-install)
  
  (with-open-file
   (stream "config.lisp" :direction :output :if-exists :supersede)
   (format stream ";;; ~A file configuration, feel free to edit~%" *prog-name*)
   (format stream ";;;  or overwrite with install scripts.~2%")
   (format stream "(defparameter *prog-name* ~S)~2%" *prog-name*)
   (format stream "(defparameter *install-dir* ~S)~2%" *install-dir*)
   (format stream "(defparameter *lisp* ~S)~2%" *lisp*)
   (format stream "(defparameter *bin-dir* ~S)~%" *bin-dir*)
   (format stream "(defparameter *lib-dir* ~S)~%" *lib-dir*)
   (format stream "(defparameter *doc-dir* ~S)~%" *doc-dir*)
   (format stream "(defparameter *man-dir* ~S)~%" *man-dir*)
   (format stream "(defparameter *share-dir* ~S)~2%" *share-dir*)
   (format stream "(defparameter *separator* ~S)~%" #+UNIX "/" #+WIN32 "\\")))



(defun compiling-files ()
  (load "config")
  (dolist (i (directory "*.lisp"))
    (format t "Compiling: ~A~%" i)
    (unless *check*
      (compile-file i))))

(defun clean-files ()
  (dolist (i (nconc (directory "*~")
		    (directory "*.fas") (directory "*.lib") (directory "*.x86f")))
    (format t "cleaning: ~A~%" i)
    (delete-file (format nil "~A" i))))

      

(defun installing-files ()
  (load "config")
  
  (format t "Creating subdirectory~%")
  (unless *check*
    (ensure-directories-exist *bin-dir*)
    (ensure-directories-exist *lib-dir*)
    (ensure-directories-exist *doc-dir*)
    (ensure-directories-exist *man-dir*)
    (ensure-directories-exist *share-dir*))


  (format t "Installing compiled files in lib directory~%")
  (dolist (i (nconc (directory "*.fas") (directory "*.lib") (directory "*.x86f")))
    (let ((dest (format nil "~A~A" *lib-dir* (file-namestring i))))
      (format t "Installing: ~A to ~A~%"  i dest)
      (unless *check*
	(copy-file i dest))))

  (format t "Creating launcher script~%")
  (unless *check*
    (with-open-file
     (stream (format nil "~A~A" *bin-dir* *launcher-name*)
	     :direction :output
	     :if-exists :supersede)
     #+UNIX (format stream "#!/bin/sh~2%")
     (format stream "~A \"~A~A\" ~A~%" *lisp* *lib-dir* *prog-bin-name*
	     #+UNIX "$*"
	     #+WIN32 "%1 %2 %3 %4 %5 %6 %7 %8 %9")))

  #+UNIX
  (progn
    (format t "Change launcher permission to executable~%")
    (unless *check*
      #+CLISP
      (shell (format nil "chmod a+x ~A~A"
		     *bin-dir* *launcher-name*))
      #+CMU
      (unix:unix-chmod (format nil "~A~A" *bin-dir* *launcher-name*) #o755)))

  

  (format t "Installing documentation to ~A~%" *doc-dir*)
  (unless *check*
    (dolist (i '("AUTHORS" "BUGS" "README" "COPYING" "INSTALL" "index.html" "clwebchat.html"))
      (let ((dest (format nil "~A~A" *doc-dir* (file-namestring i))))
	(format t "Installing: ~A to ~A~%"  i dest)
	(unless *check*
	  (copy-file i dest)))))
  
  (format t "Installing man page to ~A~%" *man-dir*)
  (unless *check*
    (dolist (i (directory "*.1"))
      (let ((dest (format nil "~A~A" *man-dir* (file-namestring i))))
	(format t "Installing: ~A to ~A~%"  i dest)
	(unless *check*
	  (copy-file i dest)))))
  
  (format t "Installing share files to ~A~%" *share-dir*)
  (unless *check*
    (dolist (i (directory "share/*.png"))
      (let ((dest (format nil "~A~A" *share-dir* (file-namestring i))))
	(format t "Installing: ~A to ~A~%"  i dest)
	(unless *check*
	  (copy-file i dest))))))



(case *action*
  (CONFIG
   (format t "configuring~%")
   (configure-files))
  (COMPILE
   (format t "compiling~%")
   (compiling-files))
  (INSTALL
   (format t "installing~%")
   (installing-files)
   (resume-install t))
  (CLEAN
   (format t "cleaning~%")
   (clean-files))
  (t (format t "Configuring, compiling and installing~%")
     (configure-files)
     (compiling-files)
     (installing-files)
     (clean-files)
     (resume-install t t)))


(quit)
