;;; --------------------------------------------------------------------------
;;; 
;;; Date: ven mai 21 14:19:24 CEST 2004
;;;
;;; --------------------------------------------------------------------------
;;;
;;; (C) 2002 Philippe Brochard <hocwp@free.fr>
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;;
;;; --------------------------------------------------------------------------


(in-package "COMMON-LISP-USER")

(defpackage MY-HTTP
  (:use COMMON-LISP COMMON-LISP-USER EXT TOOLS)
  (:export "GET-HTTP"
	   "FIND-HTTP-HEADERS"
	   "SEND-HTTP"
	   "SEND-FILE-HTTP"
	   "FIND-FILE-MIME-TYPE"
	   "CLEAN-HTTP-CONTENT"
	   "WRITE-HTTP-CONTENT"))

(in-package "MY-HTTP")




(defvar *http-mime-types*
  '(("html" "text/html; charset=iso-8859-1")
    ("txt"  "text/plain; charset=iso-8859-1")
    ("jpg"  "image/jpeg")
    ("jpeg" "image/jpeg")
    ("gif"  "image/gif")
    ("png"  "image/png")
    ("tif"  "image/tiff")
    ("tiff" "image/tiff")
    ("css"  "text/css")
    ("gz"   "application/octet-stream")
    ("ps"   "application/postscript")
    ("pdf"  "application/pdf")
    ("eps"  "application/postscript")
    ("tar"  "application/x-tar")
    ("rpm"  "application/x-rpm")
    ("zip"  "application/zip")
    ("mp3"  "audio/mpeg")
    ("mp2"  "audio/mpeg")
    ("mid"  "audio/midi")
    ("midi" "audio/midi")
    ("wav"  "audio/x-wav")
    ("au"   "audio/basic")
    ("ram"  "audio/pn-realaudio")
    ("ra"   "audio/x-realaudio")
    ("mpg"  "video/mpeg")
    ("mpeg" "video/mpeg")
    ("qt"   "video/quicktime")
    ("mov"  "video/quicktime")
    ("avi"  "video/x-msvideo")
    ("xul"  "application/vnd.mozilla.xul+xml")
    ("ico"  "image/x-icon")))



(defun parse-a-get/head-url (url type)
  (setq url (string-to-list url :split-char #\?))
  (list type
	(first url)
	(mapcar #'(lambda (x)
		    (string-to-list x :split-char #\=))
		(string-to-list (second url) :split-char #\&))))

    
(defun parse-a-post-url (sock length)
  (do ((i 0 (1+ i))
       (c)
       (word1)
       (word2)
       (res))
      ((>= i length)
       (when word1
	 (push (coerce (reverse word1) 'string) word2))
       (push (reverse word2) res)
       (list (reverse res)))
    (setq c (read-char sock))
    (case c
      (#\& (when word1
	     (push (coerce (reverse word1) 'string) word2))
       (push (reverse word2) res)
       (setq word1 ())
       (setq word2 ()))
      (#\= (when word1
	     (push (coerce (reverse word1) 'string) word2))
       (setq word1 ()))
      (t (push c word1)))))





(defun write-content-to-file (sock length filename)
  (with-open-file
   (stream filename :direction :output
	   :if-exists :supersede
	   :element-type #+CLISP '(unsigned-byte 8) #+CMU 'character)
   #+CLISP (setf (stream-element-type sock) '(unsigned-byte 8))
   (transfert-stream sock stream length)
   #+CLISP (setf (stream-element-type sock) 'character))
  (format nil "Output to ~A (~A)" filename length))

(defun return-content-in-header (sock length)
  (let* ((data (make-array length
			   :element-type #+CLISP '(unsigned-byte 8)
			   #+CMU 'character)))
    #+CLISP (setf (stream-element-type sock) '(unsigned-byte 8))
    (read-sequence data sock)
    #+CLISP (setf (stream-element-type sock) 'character)
    data))


(defun get-http-content (sock length filename)
  (cond (filename (write-content-to-file sock length filename))
	(t (return-content-in-header sock length))))



(defun read-line-from-sock (sock)
  (string-trim '(#\return #\linefeed #\newline)
	       (read-line sock nil :eof)))

(defun extract-content (filename savedir)
  (with-open-file
   (istream filename :direction :input
	    :element-type 'character)
   (let ((len (file-length istream))
	 (bound-len (length (read-line-from-sock istream)))
	 (savename ""))
     (do ((line (read-line-from-sock istream)
		(read-line-from-sock istream)))
	 ((or (eq line :eof) (string= line "")))
       (when (string= (first (string-to-list line)) "Content-Disposition:")
	 (multiple-value-bind
	     (pos1 pos2)
	     (find-string "filename=\"" line
			  :test #'(lambda (str pos1 pos2)
				    (when (and pos1 pos2 (< pos1 pos2))
				      (multiple-value-bind
					  (npos1)
					  (find-string "\"" str :start pos2)
					(values pos2 npos1)))))
	   (when (and pos1 pos2 (< pos1 pos2))
	     (setq savename (file-namestring (subseq line pos1 pos2)))))))
     (unless (string= savename "")
       (with-open-file
	(ostream (concatenate 'string savedir savename) :direction :output
		 :element-type #+CLISP '(unsigned-byte 8)
		 #+CMU 'character
		 :if-exists :supersede)
	#+CLISP (setf (stream-element-type istream) '(unsigned-byte 8))
	(transfert-stream istream ostream (- len bound-len (file-position istream) 6))
	#+CLISP (setf (stream-element-type istream) 'character))
       savename))))

(defun parse-a-post-url-multipart (sock length tempfile savedir)
  (let ((tempfile (concatenate 'string savedir tempfile))
	(savename nil))
    (write-content-to-file sock length tempfile)
    (setq savename (extract-content tempfile savedir))
    (delete-file tempfile)
    (list savename)))


(defun read-line-list-from-sock (sock)
  (string-to-list (string-trim '(#\return #\linefeed #\newline)
			       (read-line sock))))


(defun get-http (sock &key (filename nil) (tempfile "temp.tmp") (savedir ""))
  "Return a list with http content '(host :GET|:POST|:CONTENT directory content-header.)
When filename is not null then write content to file 'filename'"
  (let (ret
	host
	directory
	(length 0)
	type)
    (do ((head (read-line-list-from-sock sock)
	       (read-line-list-from-sock sock)))
	((null head))
      (case (intern (string-upcase (first head)) "MY-HTTP")
	(GET (setq ret (parse-a-get/head-url (second head) :GET)))
	(HEAD (setq ret (parse-a-get/head-url (second head) :HEAD)))
	(POST (setq ret '(post))
	      (setq directory (second head)))
	(|HOST:| (push (second head) host))
	(|CONTENT-LENGTH:| (setq length (parse-integer (second head))))
	(|CONTENT-TYPE:| (setq type (second head)))))
    (when (eq (first ret) 'post)
      (if (string= type "multipart/form-data;")
	  (setq ret (parse-a-post-url-multipart sock length tempfile savedir))
	(setq ret (parse-a-post-url sock length)))
      (push directory ret)
      (push :POST ret))
    (unless ret
      (push (get-http-content sock length filename)  ret)
      (push directory ret)
      (push :CONTENT ret))
    (push (car host) ret)
    ret))



(defun find-http-headers (headers str)
  "search a string in headers and return its value"
  (dolist (head (fourth headers))
    (when (string-equal (car head) str)
      (return-from find-http-headers (second head)))))



(defun send-http (sock content-type content &optional only-head)
  (let* ((len (length content))
	 (data #+CLISP (make-array len :element-type '(unsigned-byte 8))
	       #+CMU content))
    #+CLISP
    (dotimes (i len)
      (setf (aref data i) (char-code (aref content i))))
    (format sock "HTTP/1.1 200 OK
Content-Type: ~A
Connection: close
Content-Length: ~A~2%" content-type len)
    (unless only-head
      #+CLISP (setf (stream-element-type sock) '(unsigned-byte 8))
      (write-sequence data sock)
      #+CLISP (setf (stream-element-type sock) 'character))))



(defun send-file-http (sock file &key content-type only-head)
  (with-open-file
   (stream file :direction :input
	   :element-type #+CLISP '(unsigned-byte 8)
	   #+CMU 'character)
   (let* ((len (file-length stream)))
     (format sock "HTTP/1.1 200 OK
Content-Type: ~A
Connection: close
Content-Length: ~A~2%"
	     (if content-type content-type (find-file-mime-type file))
	     len)
     (unless only-head
       #+CLISP (setf (stream-element-type sock) '(unsigned-byte 8))
       (transfert-stream stream sock len)
       #+CLISP (setf (stream-element-type sock) 'character)))))



(defun find-file-mime-type (file)
  (let ((ext (car (last (string-to-list file :split-char #\.)))))
    ;;    (format t "Extension: ~A     " ext)
    (dolist (i *http-mime-types*)
      (when (string-equal ext (car i))
	;;	(format t "Mime type: ~A~%" (second i))
	(return-from find-file-mime-type (second i)))))
  ;;  (format t "Default Mime type: ~A~%" "text/plain; charset=iso-8859-1")
  "text/plain; charset=iso-8859-1")


(defun clean-http-content (content)
  (let ((ret ())
	c)
    (dotimes (i (length content))
      (setq c (aref content i))
      (case c
	(#\+ (setq c #\Space))
	(#\% (setq c (ignore-errors (code-char 
				     (parse-integer
				      (format nil "~A~A" (aref content (1+ i))
					      (aref content (+ i 2)))
				      :radix 16))))
	 (incf i 2)
	 (case c
	   (#\Return (setq c nil))
	   (#\Newline (setq c nil)
	    (push #\Linefeed ret)))))
      (when c
	(push c ret)))
    (coerce (nreverse ret) 'string)))


(defun write-http-content (headers filename)
  (unless (eql (second headers) :CONTENT)
    (format t "Sorry, headers content no data~%")
    (return-from write-http-content))
  (let* ((content (fourth headers))
	 #+CLISP (len (length content))
	 (data #+CLISP (make-array len :element-type 'character)
	       #+CMU content))
    #+CLISP
    (dotimes (j len)
      (setf (aref data j) (code-char (aref content j))))
    (with-open-file
     (stream filename :direction :output
	     :if-exists :supersede
	     :element-type 'character)
     (write-sequence data stream))))

